#!/usr/bin/env python
######################################################################
# automatically generate a html page which lists kernel bugs the
# kernel team might want to look at:
#
# http://qa.ubuntu.com/reports/ogasawara/kernel-buglist.html
#
# this script requires that you pass in a separate file containing
# the list of bugs.  The list must be in the following format:
#
# <bug number>,<comment>,<ommit>,
#
# for example:
# 112233,kernel config change,,
# 445566,patch upstream,,
# 0,end of the current week's bug list,,
# 778899,bitesize patch,,
# 92953,test bug,ommit,
#
# The resulting display will be sorted by Importance and then
# subsorted by Status.  Bugs which come after the line
# "0,end of the current week's bug list,," will automatically fall
# under the Carried Forward section unless the "ommit" string is
# specified in the 3rd field.
######################################################################
import time
import commands
import inspect
import urllib
import sys, os, re
import BugList
from launchpadlib.launchpad import EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from lp_login import lp_login

class KernelBugList():
	launchpad = lp_login(sys.argv[0], EDGE_SERVICE_ROOT)

	longnames = {
		"linux (Ubuntu)":[
			"linux (Ubuntu Lucid)",
			"linux (Ubuntu Karmic)",
			"linux (Ubuntu Jaunty)",
			"linux (Ubuntu Intrepid)",
			"linux (Ubuntu Hardy)"],
		"linux-meta (Ubuntu)":[
			"linux-meta (Ubuntu Lucid)",
			"linux-meta (Ubuntu Karmic)",
			"linux-meta (Ubuntu Jaunty)",
			"linux-meta (Ubuntu Intrepid)",
			"linux-meta (Ubuntu Hardy)"],
		"linux-restricted-modules (Ubuntu)":[
			"linux-restricted-modules (Ubuntu Jaunty)",
			"linux-restricted-modules (Ubuntu Intrepid)"],
		"linux-restricted-modules-2.6.24 (Ubuntu)":[
			"linux-restricted-modules-2.6.24 (Ubuntu Hardy)"],
		"linux-backports-modules-2.6.28 (Ubuntu)":[
			"linux-backports-modules-2.6.28 (Ubuntu Jaunty)"],
		"linux-backports-modules-2.6.27 (Ubuntu)":[
			"linux-backports-modules-2.6.27 (Ubuntu Intrepid)"],
		"linux-backports-modules-2.6.24 (Ubuntu)":[
			"linux-backports-modules-2.6.24 (Ubuntu Hardy)"],
		"linux-firmware (Ubuntu)": [
			"linux-firmware (Ubuntu Lucid)",
			"linux-firmware (Ubuntu Karmic)",
			"linux-firmware (Ubuntu Jaunty)",
			"linux-firmware (Ubuntu Intrepid)"],
		"initramfs-tools (Ubuntu)":[
			"initramfs-tools (Ubuntu Lucid)",
			"initramfs-tools (Ubuntu Karmic)",
			"initramfs-tools (Ubuntu Jaunty)",
			"initramfs-tools (Ubuntu Intrepid)",
			"initramfs-tools (Ubuntu Hardy)"],
		"module-init-tools (Ubuntu)": [
			"module-init-tools (Ubuntu Lucid)",
			"module-init-tools (Ubuntu Karmic)",
			"module-init-tools (Ubuntu Jaunty)",
			"module-init-tools (Ubuntu Intrepid)",
			"module-init-tools (Ubuntu Hardy)"],
		"linux-ec2 (Ubuntu)":[
			"linux-ec2 (Ubuntu Lucid)",
			"linux-ec2 (Ubuntu Karmic)",
			"linux-ec2 (Ubuntu Jaunty)",
			"linux-ec2 (Ubuntu Intrepid)",
			"linux-ec2 (Ubuntu Hardy)"],
		"bcmwl (Ubuntu)":[
			"bcmwl (Ubuntu Lucid)",
			"bcmwl (Ubuntu Karmic)"],
		"linux-ubuntu-modules-2.6.24 (Ubuntu)": [
			"linux-ubuntu-modules-2.6.24 (Ubuntu Hardy)"]}

	def ParseTags(self, task):
		tag = []
		html_row = "<tr>\n"
		if "ct-rev" in task.bug.tags:
			return (tag, "")
		if "bug-watch" in task.bug.tags:
			html_row = '<tr bgcolor="#e0e0e0">'
			tag.append("bug-watch")
		if "regression-update" in task.bug.tags or "regression-release" in task.bug.tags or "regression-potential" in task.bug.tags or "regression-proposed" in task.bug.tags:
			html_row = '<tr bgcolor="#e0e0e0">'
			if "regression-update" in task.bug.tags:
				tag.append("regression-update")
			if "regression-potential" in task.bug.tags:
				tag.append("regression-potential")
			if "regression-release" in task.bug.tags:
				tag.append("regression-release")
			if "regression-proposed" in task.bug.tags:
				tag.append("regression-proposed")
		return (tag, html_row)

	def ParseNomination(self, bug, nomination, task):
		milestone = ""
		for task_temp in bug.bug_tasks:
			if (task_temp.bug_target_name in self.longnames[task.bug_target_name]) and task_temp.status != "Won't Fix" and task_temp.status != "Invalid":
				nom = re.search("\(.* (.*)\)", task_temp.bug_target_name)
				if nom != "None":
					nomination = nom.group(1)
				if task_temp.milestone:
					milestone = task_temp.milestone.name
				return (task_temp, nomination, milestone)
		return (task, nomination, milestone)

	def LoopBugTasks(self, bug, nomination, milestone):
		task = ""
		for task_temp in bug.bug_tasks:
			if task_temp.bug_target_name in self.longnames.keys():
				(task, nomination, milestone) = self.ParseNomination(bug, nomination, task_temp)
				if task.status == "Won't Fix" or task.status == "Invalid":
					continue
				return (task, nomination, milestone)
		return (task, nomination, milestone)

	def BuildHTMLRow(self, URL, num, task, milestone, nomination, tag, comment):
		html_row = "\t<td class=\"first\"><a href=\"%s\">%s</a></td>\n" % (URL, num)

		importance_color = BugList.ColorImportance(task.importance)
		html_row += "\t<td bgcolor =\"%s\">%s</td>\n" % (importance_color, task.importance)

		status_color = BugList.ColorStatus(task.status)
		html_row += "\t<td bgcolor=\"%s\">%s</td>\n" % (status_color, task.status)
		html_row += "\t<td>%s</td>\n" % (milestone)
		html_row += "\t<td>%s</td>\n" % (nomination)
		html_row += "\t<td>%s</td>\n" % (task.bug.title)

		assignee = ""
		if task.assignee_link:
			assignee = task.assignee.name
		html_row += "\t<td>%s</td>\n" % (assignee)

		if len(tag) > 0 :
			tags = "<b>%s</b> " % (" ".join(tag))
		else:
			tags = ""

		package = ""
		if "linux" not in task.bug_target_name:
			package = task.bug_target_display_name.encode('utf-8')

		html_row += "\t<td>%s %s%s</td>\n" % (package, tags, comment)
		html_row += "</tr>\n"

		return html_row

	def NominatedAndMilestoned(self, fixed):
		# This is all a hack until launchpadlib bug 322601 is fixed.
		# At that point we should be able to use something like:
		# distro_series = ubuntu.getSeries(name_or_version="hardy")
		# linux = distro_series.getSourcePackae("linux")
		# linux.searchTasks(milestone=)

		milestones = {
					"lucid-alpha-1": "21443",
					"lucid-alpha-2": "21444",
					"lucid-alpha-3": "21445",
					"ubuntu-10.04-beta-1": "21446",
					"ubuntu-10.04-beta-2": "21447",
					"ubuntu-10.04": "21439",
					"lucid-updates": "21448",
					"ubuntu-9.10": "12698",
					"karmic-updates": "12716",
					"ubuntu-9.04": "2202",
					"jaunty-updates": "2973",
					"intrepid-updates": "2134",
					"ubuntu-8.04.3": "2132",
					"ubuntu-8.04.4": "2133",
					"dapper-updates": "135",}

		ubuntu = self.launchpad.distributions['ubuntu']
		html = ""
		milestone_cnt = 0
		for target in ubuntu.series_collection:
			if not target.active:
				continue
			for milestone in target.active_milestones_collection:
				content = ""
				for pkg in self.longnames.keys():
					pkg = pkg.split(" ")
					pkg = pkg[0]
					if milestone.name in milestones.keys():
						url = "https://bugs.edge.launchpad.net/ubuntu/" + target.name + "/+source/" + pkg + "/+bugs-text?field.status%3Alist=NEW&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&field.status%3Alist=CONFIRMED&field.status%3Alist=TRIAGED&field.status%3Alist=INPROGRESS&field.status%3Alist=FIXCOMMITTED&field.milestone%3Alist=" + milestones[milestone.name]
						content = urllib.urlopen(url).readlines()
					else:
						print target.name, " not in milestones"
						continue
					for bug_num in content:
						bug_num = bug_num.strip()
						bug_task = self.launchpad.load("https://api.edge.launchpad.net/beta/ubuntu/%s/+source/%s/+bug/%s" % (target.name, pkg, bug_num))
						URL = "https://bugs.edge.launchpad.net/ubuntu/" + target.name + "/+source/" + pkg + "/+bug/" + bug_num
						html_temp = self.BuildHTMLRow(URL, bug_num, bug_task, milestone.name, target.name, "", "")
						if bug_task.status == "Fix Released" or bug_task.status == "Fix Committed":
							fixed.SortImportance(html_temp, bug_task.importance, bug_task.status, fixed)
						else:
							html += html_temp
							milestone_cnt += 1

		return (html, milestone_cnt)

	def GenerateHTML(self, buglist_data):
		list = ["regression-potential", "regression-update", "regression-release", "regression-proposed", "milestoned", "current", "carry_fwd", "inprogress", "fixed"]
		html_table = '\n<table class="sortable listing" id="trackers" border=1>\n'
		html_table += "<thead><tr>\n"
		html_table += "\t<th align=center>Bug</defanghtml_hd>\n"
		html_table += "\t<th align=center>Importance</th>\n"
		html_table += "\t<th align=center>Status</th>\n"
		html_table += "\t<th align=center>Milestone</th>\n"
		html_table += "\t<th align=center>Nomination</th>\n"
		html_table += "\t<th align=center>Summary</th>\n"
		html_table += "\t<th align=center>Assignee</th>\n"
		html_table += "\t<th align=center>Comments</th>\n"
		html_table += "</tr></thead><tbody>\n"
		html = ""
		for key in list:
			if key == "milestoned":
				html_temp = buglist_data[key]["list"]
				html_header = (buglist_data[key]["header"])
			else:
				(html_temp, size) = BugList.BuildBugList(buglist_data[key]["list"])
				html_header = (buglist_data[key]["header"] % size)
			html += html_header + html_table
			html += html_temp
			html += "</tbody></table>\n"
			html += "<p></p>\n"
		html += "</tbody></table></body></html>\n"

		return html

	def WriteFile(self, html):
		date = commands.getoutput('date -u +"%b %e %G %k:%M %Z"')
		templatefile = open ('/home/ogasawara/ubuntu-qa-tools/bug-report-framework/mult-tables-template.html', 'r')
		htmlfile = "./kernel-buglist-temp.html"
		datafile = open(htmlfile, 'w')

		for line in templatefile:
			if line.strip() == "<!-- *** Title Space *** -->":
				datafile.write("Ubuntu Kernel Bug List\n")
			elif line.strip() == "<!-- *** Header Space *** -->":
				datafile.write("<h1><a href=\"%s\">Ubuntu Kernel Bug List</a> - %s</h1>\n" % ("http://qa.ubuntu.com/reports/ogasawara/kernel-buglist-archive/kernel-buglist-description.html", date))
			elif line.strip() == "<!-- *** Table Space *** -->":
				datafile.write(html.encode('utf-8'))
			else:
				datafile.write(line)
		templatefile.close()
		datafile.close()

		return

	def Create(self):
		stats = {"New": 0,
			"Incomplete": 0,
			"Confirmed": 0,
			"Triaged": 0,
			"In Progress": 0,
			"Fix Committed": 0,
			"Fix Released": 0,
			"Invalid": 0,
			"Won\'t Fix": 0}

		current = BugList.SortedList()
		carry_fwd = BugList.SortedList()
		fixed = BugList.SortedList()
		inprogress = BugList.SortedList()
		regression_potential = BugList.SortedList()
		regression_update = BugList.SortedList()
		regression_release = BugList.SortedList()
		regression_proposed= BugList.SortedList()

		buglist = sys.argv[1]

		html_carryfwd = ""

		current_list = 1
#		current_cnt = 0
#		fixed_cnt = 0
#		carry_fwd_cnt = 0
#		inprogress_cnt = 0
#		regressions_cnt = 0
#		bug_watch_cnt = 0

		(milestoned, milestone_cnt) = self.NominatedAndMilestoned(fixed)
		uniq_list = []

		regressions = {"regression-potential": regression_potential,
						"regression-update": regression_update,
						"regression-release": regression_release,
						"regression-proposed": regression_proposed,}

		for regression in regressions.keys():
			print "Processing", regression
			linux = self.launchpad.distributions['ubuntu'].getSourcePackage(name='linux')
			collection = linux.searchTasks(tags=[regression], order_by="-importance")
			for bug_task in collection:
				print bug_task.bug.id
				URL = "https://bugs.launchpad.net/ubuntu/+source/linux/+bug/%d?comments=all" % bug_task.bug.id
				(tag, html_row) = self.ParseTags(bug_task)
				html_row += self.BuildHTMLRow(URL, str(bug_task.bug.id), bug_task, "", "", tag, "")
				regressions[regression].SortImportance(html_row, bug_task.importance, bug_task.status, regressions[regression])

		LIST = open(buglist, "r")
		for line in LIST:
			print line

			if re.match("#", line):
				continue

			arg = line.split(",")
			num = arg[0]
			comment = arg[1]
			ommit = arg[2].strip()
			if ommit == "ommit":
				print "Ommit %s" % num
				continue
			nomination = ""
			milestone = ""

			if num in uniq_list:
				print "Bug %s already exists in list.  Skipping." % (num)
				continue
			else:
				uniq_list.append(num)

			if num == "0":
				print "End of current week's list"
				current_list = 0
				continue

			URL = "https://bugs.launchpad.net/ubuntu/+bug/%s?comments=all" % num
			retry = 0
			while retry < 2:
				try:
					bug = self.launchpad.bugs[num]
					if bug.duplicate_of:
						print "Duplicate of " , bug.duplicate_of.id
						bug = bug.duplicate_of
						URL = "https://bugs.launchpad.net/ubuntu/+bug/%d?comments=all" % bug.id
					(task, nomination, milestone) = self.LoopBugTasks(bug, nomination, milestone)
					if not task:
						print "**** NOT A KERNEL BUG ****"
						break

					(tag, html_row) = self.ParseTags(task)

					if html_row == "":
#						print "ct-rev"
						break
					stats[task.status] += 1
					pkg = re.search("(^\S+)",task.bug_target_name)
					URL = "https://bugs.launchpad.net/ubuntu/+source/%s/+bug/%s?comments=all" % (pkg.group(1), bug.id)
					html_row += self.BuildHTMLRow(URL, str(bug.id), task, milestone, nomination, tag, comment)

					if current_list == 1:
						current.SortImportance(html_row, task.importance, task.status, current)
#						current_cnt = current_cnt + 1

#					if task.status != "Fix Released" and task.status != "Fix Committed" and task.status != "Invalid" and task.status != "Won\'t Fix" and tag:
#						regressions.SortImportance(html_row, task.importance, task.status, regressions)
#						if "bug-watch" in task.bug.tags:
#							bug_watch_cnt = bug_watch_cnt + 1
#						elif ("regression-potential" in task.bug.tags or "regression-update" in task.bug.tags or "regression-release" in task.bug.tags or "regression-proposed" in task.bug.tags):
#							regressions_cnt = regressions_cnt + 1

					if task.status == "Fix Released" or task.status == "Fix Committed":
						fixed.SortImportance(html_row, task.importance, task.status, fixed)
#						fixed_cnt = fixed_cnt + 1
					elif (not current_list) and task.status == "In Progress" or task.status == "Incomplete":
						inprogress.SortImportance(html_row, task.importance, task.status, inprogress)
#						inprogress_cnt = inprogress_cnt + 1
					elif (not current_list) and task.status != "Won\'t Fix" and task.status != "Invalid":
						carry_fwd.SortImportance(html_row, task.importance, task.status, carry_fwd)
#						carry_fwd_cnt = carry_fwd_cnt + 1
					retry = 2

				except:
					retry=5
					continue
					raise
					print "retry bug %s after 10sec sleep . . ." % num
					time.sleep(10)
					retry = retry + 1

		LIST.close()

		buglist_data = {"regression-potential":
							{"header": "<h1>%d Regression Potential Bugs</h1>",
							"list": regression_potential},
						"regression-update":
							{"header": "<h1>%d Regression Update Bugs</h1>",
							"list": regression_update},
						"regression-release":
							{"header": "<h1>%d Regression Release Bugs</h1>",
							"list": regression_release},
						"regression-proposed":
							{"header": "<h1>%d Regression Proposed</h1>",
							"list": regression_proposed},
						"milestoned":
							{"header": "<h1>%d Bugs Nominated and Milestoned</h1>" % milestone_cnt,
							"list": milestoned},
						"current":
							{"header": "<h1>%d Bugs Added This Week</h1>",
							"list": current},
						"carry_fwd":
							{"header": "<h1>%d Bugs Carried Forward from Previous Week(s)</h1>",
							"list": carry_fwd},
						"inprogress":
							{"header": "<h1>%d Bugs In Progress or Incomplete</h1>",
							"list": inprogress},
						"fixed":
							{"header": "<h1>%d Fixed Bugs</h1>",
							"list": fixed}
						}

#		if bug_watch_cnt > 1:
#			buglist_data["regressions"]["header"] = "<h1>%d Regressions, %d Bug Watches</h1>" % (regressions_cnt, bug_watch_cnt)
#		elif bug_watch_cnt == 1:
#			 buglist_data["regressions"]["header"] = "<h1>%d Regressions, %d Bug Watch</h1>" % (regressions_cnt, bug_watch_cnt)

		return (buglist_data, stats)
