#!/usr/bin/env python
######################################################################
# automatically generate a html page which lists kernel bugs the
# kernel team might want to look at:
#
# http://qa.ubuntu.com/reports/ogasawara/kernel-buglist.html
#
# this script requires that you pass in a separate file containing
# the list of bugs.  The list must be in the following format:
#
# <bug number>,<comment>,<ommit>,
#
# for example:
# 112233,kernel config change,,
# 445566,patch upstream,,
# 0,end of the current week's bug list,,
# 778899,bitesize patch,,
# 92953,test bug,ommit,
#
# The resulting display will be sorted by Importance and then
# subsorted by Status.  Bugs which come after the line
# "0,end of the current week's bug list,," will automatically fall
# under the Carried Forward section unless the "ommit" string is
# specified in the 3rd field.
######################################################################
import time
import commands
import inspect
import urllib
import sys, os, re
import BugList
from launchpadlib.launchpad import EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from lp_login import lp_login

class KernelBugList():
    launchpad = lp_login(sys.argv[0], EDGE_SERVICE_ROOT)

    longnames = {
        "linux (Ubuntu)":[
            "linux (Ubuntu Maverick)",
            "linux (Ubuntu Lucid)",
            "linux (Ubuntu Karmic)",
            "linux (Ubuntu Jaunty)",
            "linux (Ubuntu Intrepid)",
            "linux (Ubuntu Hardy)"],
        "linux-meta (Ubuntu)":[
            "linux-meta (Ubuntu Maverick)",
            "linux-meta (Ubuntu Lucid)",
            "linux-meta (Ubuntu Karmic)",
            "linux-meta (Ubuntu Jaunty)",
            "linux-meta (Ubuntu Intrepid)",
            "linux-meta (Ubuntu Hardy)"],
        "linux-restricted-modules (Ubuntu)":[
            "linux-restricted-modules (Ubuntu Jaunty)",
            "linux-restricted-modules (Ubuntu Intrepid)"],
        "linux-restricted-modules-2.6.24 (Ubuntu)":[
            "linux-restricted-modules-2.6.24 (Ubuntu Hardy)"],
        "linux-backports-modules-2.6.28 (Ubuntu)":[
            "linux-backports-modules-2.6.28 (Ubuntu Jaunty)"],
        "linux-backports-modules-2.6.27 (Ubuntu)":[
            "linux-backports-modules-2.6.27 (Ubuntu Intrepid)"],
        "linux-backports-modules-2.6.24 (Ubuntu)":[
            "linux-backports-modules-2.6.24 (Ubuntu Hardy)"],
        "linux-firmware (Ubuntu)": [
            "linux-firmware (Ubuntu Maverick)",
            "linux-firmware (Ubuntu Lucid)",
            "linux-firmware (Ubuntu Karmic)",
            "linux-firmware (Ubuntu Jaunty)",
            "linux-firmware (Ubuntu Intrepid)"],
        "initramfs-tools (Ubuntu)":[
            "initramfs-tools (Ubuntu Maverick)",
            "initramfs-tools (Ubuntu Lucid)",
            "initramfs-tools (Ubuntu Karmic)",
            "initramfs-tools (Ubuntu Jaunty)",
            "initramfs-tools (Ubuntu Intrepid)",
            "initramfs-tools (Ubuntu Hardy)"],
        "module-init-tools (Ubuntu)": [
            "module-init-tools (Ubuntu Maverick)",
            "module-init-tools (Ubuntu Lucid)",
            "module-init-tools (Ubuntu Karmic)",
            "module-init-tools (Ubuntu Jaunty)",
            "module-init-tools (Ubuntu Intrepid)",
            "module-init-tools (Ubuntu Hardy)"],
        "linux-ec2 (Ubuntu)":[
            "linux-ec2 (Ubuntu Lucid)",
            "linux-ec2 (Ubuntu Karmic)",
            "linux-ec2 (Ubuntu Jaunty)",
            "linux-ec2 (Ubuntu Intrepid)",
            "linux-ec2 (Ubuntu Hardy)"],
        "bcmwl (Ubuntu)":[
            "bcmwl (Ubuntu Maverick)",
            "bcmwl (Ubuntu Lucid)",
            "bcmwl (Ubuntu Karmic)"],
        "linux-ubuntu-modules-2.6.24 (Ubuntu)": [
            "linux-ubuntu-modules-2.6.24 (Ubuntu Hardy)"]}

    def ParseTags(self, task):
        tag = []
        html_row = "<tr>\n"
        if "ct-rev" in task.bug.tags:
            return (tag, "")
        if "bug-watch" in task.bug.tags:
            html_row = '<tr bgcolor="#e0e0e0">'
            tag.append("bug-watch")
        if "regression-update" in task.bug.tags or "regression-release" in task.bug.tags or "regression-potential" in task.bug.tags or "regression-proposed" in task.bug.tags:
            html_row = '<tr bgcolor="#e0e0e0">'
            if "regression-update" in task.bug.tags:
                tag.append("regression-update")
            if "regression-potential" in task.bug.tags:
                tag.append("regression-potential")
            if "regression-release" in task.bug.tags:
                tag.append("regression-release")
            if "regression-proposed" in task.bug.tags:
                tag.append("regression-proposed")
        return (tag, html_row)

    def ParseNomination(self, bug, nomination, task):
        milestone = ""
        for task_temp in bug.bug_tasks:
            if (task_temp.bug_target_name in self.longnames[task.bug_target_name]) and task_temp.status != "Won't Fix" and task_temp.status != "Invalid":
                nom = re.search("\(.* (.*)\)", task_temp.bug_target_name)
                if nom != "None":
                    nomination = nom.group(1)
                if task_temp.milestone:
                    milestone = task_temp.milestone.name
                return (task_temp, nomination, milestone)
        return (task, nomination, milestone)

    def LoopBugTasks(self, bug, nomination, milestone):
        task = ""
        for task_temp in bug.bug_tasks:
            if task_temp.bug_target_name in self.longnames.keys():
                (task, nomination, milestone) = self.ParseNomination(bug, nomination, task_temp)
                if task.status == "Won't Fix" or task.status == "Invalid":
                    continue
                return (task, nomination, milestone)
        return (task, nomination, milestone)

    def BuildHTMLRow(self, URL, num, task, milestone, nomination, tag, comment):
        html_row = "\t<td class=\"first\"><a href=\"%s\">%s</a></td>\n" % (URL, num)

        importance_color = BugList.ColorImportance(task.importance)
        html_row += "\t<td bgcolor =\"%s\">%s</td>\n" % (importance_color, task.importance)

        status_color = BugList.ColorStatus(task.status)
        html_row += "\t<td bgcolor=\"%s\">%s</td>\n" % (status_color, task.status)
        html_row += "\t<td>%s</td>\n" % (milestone)
        html_row += "\t<td>%s</td>\n" % (nomination)
        html_row += "\t<td>%s</td>\n" % (task.bug.title)

        assignee = ""
        if task.assignee_link:
            assignee = task.assignee.name
        html_row += "\t<td>%s</td>\n" % (assignee)

        if len(tag) > 0 :
            tags = "<b>%s</b> " % (" ".join(tag))
        else:
            tags = ""

        package = ""
        if "linux" not in task.bug_target_name:
            package = task.bug_target_display_name.encode('utf-8')

        html_row += "\t<td>%s %s%s</td>\n" % (package, tags, comment)
        html_row += "</tr>\n"

        return html_row

    def NominatedAndMilestoned(self, closed):
        # This is all a hack until launchpadlib bug 322601 is fixed.
        # At that point we should be able to use something like:
        # distro_series = ubuntu.getSeries(name_or_version="hardy")
        # linux = distro_series.getSourcePackae("linux")
        # linux.searchTasks(milestone=)

        milestones = {
                    "lucid-alpha-1": "21443",
                    "lucid-alpha-2": "21444",
                    "lucid-alpha-3": "21445",
                    "ubuntu-10.04-beta-1": "21446",
                    "ubuntu-10.04-beta-2": "21447",
                    "ubuntu-10.04": "21439",
                    "lucid-updates": "21448",
                    "ubuntu-9.10": "12698",
                    "karmic-updates": "12716",
                    "ubuntu-9.04": "2202",
                    "jaunty-updates": "2973",
                    "intrepid-updates": "2134",
                    "ubuntu-8.04.3": "2132",
                    "ubuntu-8.04.4": "2133",
                    "dapper-updates": "135",}

        ubuntu = self.launchpad.distributions['ubuntu']
        html = ""
        milestone_cnt = 0
        for target in ubuntu.series_collection:
            if not target.active:
                continue
            for milestone in target.active_milestones_collection:
                content = ""
                for pkg in self.longnames.keys():
                    pkg = pkg.split(" ")
                    pkg = pkg[0]
                    if milestone.name in milestones.keys():
                        url = "https://bugs.edge.launchpad.net/ubuntu/" + target.name + "/+source/" + pkg + "/+bugs-text?field.status%3Alist=NEW&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&field.status%3Alist=CONFIRMED&field.status%3Alist=TRIAGED&field.status%3Alist=INPROGRESS&field.status%3Alist=FIXCOMMITTED&field.milestone%3Alist=" + milestones[milestone.name]
                        content = urllib.urlopen(url).readlines()
                    else:
                        print target.name, " not in milestones"
                        continue
                    for bug_num in content:
                        bug_num = bug_num.strip()
                        bug_task = self.launchpad.load("https://api.edge.launchpad.net/beta/ubuntu/%s/+source/%s/+bug/%s" % (target.name, pkg, bug_num))
                        URL = "https://bugs.edge.launchpad.net/ubuntu/" + target.name + "/+source/" + pkg + "/+bug/" + bug_num
                        html_temp = self.BuildHTMLRow(URL, bug_num, bug_task, milestone.name, target.name, "", "")
                        if bug_task.status == "Fix Released" or bug_task.status == "Invalid" or bug_task.status == "Won't Fix":
                            closed.SortImportance(html_temp, bug_task.importance, bug_task.status, closed)
                        else:
                            html += html_temp
                            milestone_cnt += 1

        return (html, milestone_cnt)

    def GenerateStatsHTML(self, stats):
        html = '<h1>Summary of Bugs</h1>\n'
        html += '<table class="sortable listing" id="trackers" border=2>\n'
        table_header = ""
        table_data = ""
        for status in ["Invalid", "Won't Fix", "Fix Released", "Fix Committed", "In Progress", "Triaged", "Confirmed", "Incomplete", "New"]:
            table_header += '<td>' + status
            table_data += '<td>' + str(stats[status])
        html += '\t<tr>\n'
        html += table_header
        html += '\n<tr>\n' + table_data
        html += '\n</table>\n<p>\n'

        return html

    def GenerateHTML(self, buglist_data):
        list = ["current", "closed"]
        html_table = '\n<table class="sortable listing" id="trackers" border=2>\n'
        html_table += "<thead><tr>\n"
        html_table += "\t<th align=center>Bug</defanghtml_hd>\n"
        html_table += "\t<th align=center>Importance</th>\n"
        html_table += "\t<th align=center>Status</th>\n"
        html_table += "\t<th align=center>Milestone</th>\n"
        html_table += "\t<th align=center>Nomination</th>\n"
        html_table += "\t<th align=center>Summary</th>\n"
        html_table += "\t<th align=center>Assignee</th>\n"
        html_table += "\t<th align=center>Comments</th>\n"
        html_table += "</tr></thead><tbody>\n"
        html = ""
        for key in list:
            (html_temp, size) = BugList.BuildBugList(buglist_data[key]["list"])
            html_header = (buglist_data[key]["header"] % size)
            html += html_header + html_table
            html += html_temp
            html += "</tbody></table>\n"
            html += "<p></p>\n"
        html += "</tbody></table></body></html>\n"

        return html

    def WriteFile(self, html):
        date = commands.getoutput('date -u +"%b %e %G %k:%M %Z"')
        templatefile = open ('/home/ogasawara/ubuntu-qa-tools/bug-report-framework/mult-tables-template.html', 'r')
        htmlfile = "./kernel-buglist-top50.html"
        datafile = open(htmlfile, 'w')

        for line in templatefile:
            if line.strip() == "<!-- *** Title Space *** -->":
                datafile.write("Ubuntu Kernel Top 50 Bug List\n")
            elif line.strip() == "<!-- *** Header Space *** -->":
                datafile.write("<h1><a href=\"%s\">Ubuntu Kernel Top 50 Bug List</a> - %s</h1>\n" % ("http://qa.ubuntu.com/reports/ogasawara/kernel-buglist-archive/kernel-buglist-description.html", date))
            elif line.strip() == "<!-- *** Table Space *** -->":
                datafile.write(html.encode('utf-8'))
            else:
                datafile.write(line)
        templatefile.close()
        datafile.close()

        return

    def Create(self):
        stats = {"New": 0,
            "Incomplete": 0,
            "Confirmed": 0,
            "Triaged": 0,
            "In Progress": 0,
            "Fix Committed": 0,
            "Fix Released": 0,
            "Invalid": 0,
            "Won\'t Fix": 0}

        current = BugList.SortedList()
        closed = BugList.SortedList()

        buglist = sys.argv[1]

        uniq_list = []

        LIST = open(buglist, "r")
        for line in LIST:
            print line

            if re.match("#", line):
                continue

            arg = line.split(",")
            num = arg[0]
            comment = arg[1]
            nomination = ""
            milestone = ""

            if num in uniq_list:
                print "Bug %s already exists in list.  Skipping." % (num)
                continue
            else:
                uniq_list.append(num)

            URL = "https://bugs.launchpad.net/ubuntu/+bug/%s?comments=all" % num
            retry = 0
            while retry < 2:
                try:
                    bug = self.launchpad.bugs[num]
                    if bug.duplicate_of:
                        print "Duplicate of " , bug.duplicate_of.id
                        bug = bug.duplicate_of
                        URL = "https://bugs.launchpad.net/ubuntu/+bug/%d?comments=all" % bug.id
                    (task, nomination, milestone) = self.LoopBugTasks(bug, nomination, milestone)
                    if not task:
                        print "**** NOT A KERNEL BUG ****"
                        break

                    (tag, html_row) = self.ParseTags(task)

                    if html_row == "":
                        break
                    stats[task.status] += 1
                    pkg = re.search("(^\S+)",task.bug_target_name)
                    URL = "https://bugs.launchpad.net/ubuntu/+source/%s/+bug/%s?comments=all" % (pkg.group(1), bug.id)
                    html_row += self.BuildHTMLRow(URL, str(bug.id), task, milestone, nomination, tag, comment)

                    print task.status
                    if task.status == "Fix Released" or task.status == "Invalid" or task.status == "Won't Fix":
                        closed.SortImportance(html_row, task.importance, task.status, closed)
                    else:
                        current.SortImportance(html_row, task.importance, task.status, current)

                    break
                except:
                    print "retry bug %s after 10sec sleep . . ." % num
                    time.sleep(10)
                    retry = retry + 1

        LIST.close()

        buglist_data = {"current":
                            {"header": "<h1>%d Open Bugs</h1>",
                            "list": current},
                        "closed":
                            {"header": "<h1>%d Closed Bugs</h1>",
                            "list": closed}
                        }
        return (buglist_data, stats)
