#!/usr/bin/env python
#
# The following creates a stacked bar plot of the total number of
# Fix Released linux kernel bugs on a per release basis.  It also showcases
# the subset of bugs that were Fix Released by a referenced git commit
# from the kernel team.  If a git commit contained a "LP:<bug#>" in the
# changelog, it triggers the Launchpad Janitor to move the bug to a
# Fix Released state.
#
# Refer to canonical-qa-tracking:misc-scripts/fixed-kernel-bugs.py which is
# responsible for examining all Fix Released linux bugs, determining which
# release they belong to, and whether the Launchpad Janitor moved the bug to
# a Fix Released state or not.  fixed-kernel-bugs.py generates both the
# "fixed-linux-known-patches.data" and "fixed-linux-total.data" files which
# are passed to this script.

import sys
import numpy as np
import matplotlib.pyplot as plt

# attach bug total labels to bars
def label_bar(kernel, total):
    count = 0
    for rect in kernel:
        height_kernel = rect.get_height()
        height_total = total[count].get_height()
        plt.text(rect.get_x()+rect.get_width()/2., 1.05*height_kernel, '%d'%int(height_kernel), ha='center', va='bottom')
        plt.text(rect.get_x()+rect.get_width()/2., 1.05*(height_total + height_kernel), '%d'%int(height_total + height_kernel), ha='center', va='bottom')
        count = count + 1

if len(sys.argv) != 3:
    print "Usage: %s <fixed_linux_known_patches.data> <fixed_linux_total.data>" % (sys.argv[0])
    sys.exit(1)

KERNEL_TEAM_DATA = open(sys.argv[1], "r")
LINUX_TOTAL_DATA = open(sys.argv[2], "r")

# holds data read from KERNEL_TEAM_DATA and LINUX_TOTAL_DATA
kernel_team_data  = []
linux_total_data = []

# holds fixed bug counts per release
kernel_team_fixed  = []
linux_total_fixed = []

# holds the labels for every xtick on the xaxis
xlabels=[]

for line in KERNEL_TEAM_DATA:
    kernel_team_data = line.split(",")

    # x-axis labels are always the first element of the line of data
    # For Example "Jaunty Alpha3,<bug#>,<bug#>,<bug#>,...
    xlabels.append(kernel_team_data[0].strip())

    # get the total number of Fix Released bugs by the kernel team
    if len(kernel_team_data) > 0:
       kernel_team_fixed.append(len(kernel_team_data) - 1)
    else:
        kernel_team_fixed.append(0)

KERNEL_TEAM_DATA.close()

count = 0
for line in LINUX_TOTAL_DATA:
    linux_total_data = line.split(",")

    # get the total number of Fix Released bugs per release
    if len(linux_total_data) > 0:
        # take into account the bars are stacked (ie kernel_team_fixed is
        # a subset of linux_total_fixed)
        linux_total_fixed.append(len(linux_total_data) - 1 - kernel_team_fixed[count])
    else:
        linux_total_fixed.append(0)
    count = count + 1

LINUX_TOTAL_DATA.close()

total_bars = len(xlabels)   # total number of bars to create
xticks = np.arange(total_bars)    # the x locations for the bars
width = .75    # the width of the bars

# create the bars
p1 = plt.bar(xticks, kernel_team_fixed, width, color='#00FF00')
p2 = plt.bar(xticks, linux_total_fixed, width, color='g', bottom=kernel_team_fixed)

# customize graph
plt.ylabel('Bugs Fixed')
plt.title('Fixed linux Bugs per Release')
plt.xticks(xticks+width/2, xlabels, rotation=270)
plt.ylim(ymax=350)
plt.yticks(np.arange(0,351,20))
plt.subplots_adjust(bottom=.30)
plt.legend( (p2[0], p1[0]), ('Total Bugs Fixed', 'Fixed with Known Patch') )
label_bar(p1, p2)
plt.savefig("fixed-kernel-bugs-bars-stacked.png")

plt.show()

