#!/usr/bin/env python
#
# The following parses through the list of Fix Released linux bugs
# and determines which release it was fixed in based on either
# date_fix_released or date_closed.  It also determines if the fix
# has a referenced git commit from the kernel team.  This is implied
# by the fact that if git commit referenced a "LP:<bug#>" in the
# changelog the Launchpad Janitor would have moved the bug into a
# Fix Released state.
#
# The script will generate two files, "fixed-linux-known-patches.data"
# and "fixed-linux-total.data".  "fixed-linux-known-patches.data" contains
# the bugs that were Fix Released by the janitor on a per release basis.
# "fixed-linux-total.data" contains all the linux bugs that were Fix Released
# on a per release basis.
#
# *.data file format is:
#
# <release>,<bug>,<bug>,<bug>,....
#
# For Example:
#
# Hardy Alpha 1
# Hardy Alpha 2,157909
# Hardy Alpha 3
# Hardy Alpha 4,140784,178343,176587,174367,175834,152689
# ...

import sys
import os
import re
import bisect
import string
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

# list of Fix Released bugs which have a specific git commit associated
# with them (separated per release)
kernelteam_fixed = [
                ["Hardy Alpha 1"],
                ["Hardy Alpha 2"],
                ["Hardy Alpha 3"],
                ["Hardy Alpha 4"],
                ["Hardy Alpha 5"],
                ["Hardy Alpha 6"],
                ["Hardy Beta"],
                ["Hardy 8.04"],
                ["Intrepid Alpha 1"],
                ["Intrepid Alpha 2"],
                ["Intrepid Alpha 3"],
                ["Intrepid Alpha 4"],
                ["Intrepid Alpha 5"],
                ["Intrepid Alpha 6"],
                ["Intrepid Beta"],
                ["Intrepid 8.10"],
                ["Jaunty Alpha 1"],
                ["Jaunty Alpha 2"],
                ["Jaunty Alpha 3"],
                ["Jaunty Alpha 4"],
                ["Jaunty Alpha 5"],
                ["Jaunty Alpha 6"],
                ["Jaunty Beta"],
                ["Jaunty 9.04"],]

# list of all Fix Released bugs (separated per release)
total_fixed = [
                ["Hardy Alpha 1"],
                ["Hardy Alpha 2"],
                ["Hardy Alpha 3"],
                ["Hardy Alpha 4"],
                ["Hardy Alpha 5"],
                ["Hardy Alpha 6"],
                ["Hardy Beta"],
                ["Hardy 8.04"],
                ["Intrepid Alpha 1"],
                ["Intrepid Alpha 2"],
                ["Intrepid Alpha 3"],
                ["Intrepid Alpha 4"],
                ["Intrepid Alpha 5"],
                ["Intrepid Alpha 6"],
                ["Intrepid Beta"],
                ["Intrepid 8.10"],
                ["Jaunty Alpha 1"],
                ["Jaunty Alpha 2"],
                ["Jaunty Alpha 3"],
                ["Jaunty Alpha 4"],
                ["Jaunty Alpha 5"],
                ["Jaunty Alpha 6"],
                ["Jaunty Beta"],
                ["Jaunty 9.04"],]

# sorted list of release dates beginning with Hardy Alpha 1
release_dates = [
                "2007-12-01", # hardy alpha1, position=0
                "2007-12-21", # hardy alpha2, position=1
                "2008-01-10", # hardy alpha3, position=2
                "2008-02-01", # hardy alpha4, position=3
                "2008-02-22", # hardy alpha5, position=4
                "2008-03-06", # hardy alpha6, position=5
                "2008-03-20", # hardy beta, position=6
                "2008-04-24", # hardy final, position=7
                "2008-06-27", # intrepid alpha1, position=8
                "2008-07-11", # intrepid alpha2, position=9
                "2008-07-24", # intrepid alpha3, position=10
                "2008-09-05", # intrepid alpha5, position=11
                "2008-09-18", # intrepid alpha6, position=12
                "2008-08-14", # intrepid alpha4, position=13
                "2008-10-02", # intrepid beta, position=14
                "2008-10-30", # intrepid final, position=15
                "2008-11-22", # jaunty alpha1, position=16
                "2008-12-19", # jaunty alpha2, position=17
                "2009-01-16", # jaunty alpha3, position=18
                "2009-02-05", # jaunty alpha4, position=19
                "2009-02-26", # jaunty alpha5, position=20
                "2009-03-12", # jaunty alpha6, position=21
                "2009-03-26", # jaunty beta, position=22
                "2009-04-23",] #jaunty final, position=23

# Cycle through all the comments of a bug to determine if the LP Janitor
# posted a comment on the same date the bug was date_fix_committed or
# date_closed.  Return an incrementing janitor_kernel_fixed count if
# the LP Janitor actually Fix Released the bug.
def parse_msgs(msgs, remaining, bug, janitor, date_fixed, janitor_kernel_fixed):
    global kernelteam_fixed
    global release_dates

    # For some reason launchpad lib only provides the first 50 messages/comments
    # per bug.  So, we have to recursively cycle though msgs.next in order to
    # examine all the comments from bugs with > 50 comments
    if remaining > 50:
        remaining = remaining - 50
        janitor_kernel_fixed = parse_msgs(msgs.next, remaining, bug, janitor, date_fixed, janitor_kernel_fixed)
        return janitor_kernel_fixed

    # Messages/comments are returned oldest to newest, so we want to start
    # looking for Janitor posted comments for newest to oldest
    for msg in reversed(msgs.entries):
        # get the owner of the comment/message and the date it was created
        owner = launchpad.load(msg["owner_link"])
        date_msg_created = msg['date_created']

        # from the timestamp, parse the date the message was created
        date = re.match("(\d\d\d\d-\d\d-\d\d).*", str(date_msg_created))
        if date == None:
            continue
        date_msg_created = date.group(1)

        # comment is newer than the date the bug was fixed, continue
        if str(date_msg_created) > str(date_fixed):
            continue
        # comment date == the date bug fixed, and commenter == janitor
        elif (str(date_msg_created) == str(date_fixed)) and (owner.name == janitor.name):
            janitor_kernel_fixed = janitor_kernel_fixed + 1
            # Determine which release the bug was fixed.
            position = bisect.bisect(release_dates, date_fixed)
            if position >= len(release_dates):
                continue
            kernelteam_fixed[position].append(str(bug.id))
            break
        # comment date == date bug fixed, but commenter != janitor
        elif str(date_msg_created) == str(date_fixed):
            continue
        # comment date is older than date fixed
        else:
            break
    return janitor_kernel_fixed

# Since we are dealing with large collections, we slice them into lists
# and operate on the lists. See https://help.launchpad.net/API/launchpadlib
def check_end(start, stop, STEP, end):
    if stop == end:
        return (end, end)
    start = stop
    stop = start + STEP
    if stop > end:
        stop = end

    return (start, stop)

# Initialize first list slice to oeprate on
def init_step(total, STEP):
    start = 0
    end = start + STEP
    if end > total:
        end = total

    return (start,end)

# Parse %Y-%m-%d from date_fix_released or date_closed timestamp
def get_date_fixed(task, err_count):
    date_fixed = str(task.date_fix_released)
    date = re.match("(\d\d\d\d-\d\d-\d\d).*", date_fixed)
    if date == None:
        date_fixed = task.date_closed
        date = re.match("(\d\d\d\d-\d\d-\d\d).*", date_fixed)
        if date == None:
            err_count = err_count + 1
            return None
    return (date.group(1), err_count)

# Write data to file
def create_file(file, fixed):
    FILE = open(file, 'w')
    for x in fixed:
        line = string.join(x,",") + "\n"
        print line
        FILE.write(line)
    FILE.close

def main ():
    global kernelteam_fixed

    ubuntu = launchpad.distributions["ubuntu"]
    collection = ubuntu.getSourcePackage(name="linux")
    janitor = launchpad.people['janitor']
    try:
        janitor_tasks = collection.searchTasks(status="Fix Released", order_by="-datecreated", omit_duplicates=True, bug_commenter=janitor)
        total_tasks = collection.searchTasks(status="Fix Released", order_by="-datecreated", omit_duplicates=True)
    except HTTPError:
        print "retry . . ." 
        janitor_tasks = collection.searchTasks(status="Fix Released", order_by="-datecreated", omit_duplicates=True, bug_commenter=janitor)
        total_tasks = collection.searchTasks(status="Fix Released", order_by="-datecreated", omit_duplicates=True)

    janitor_fix_released = int(janitor_tasks._wadl_resource.representation['total_size'])
    total_fix_released = int(total_tasks._wadl_resource.representation['total_size'])

    STEP = 100  # size of collection list slice

    (start, end) = init_step(total_fix_released, STEP) # first list slice
    err_count = 0

    # Cycle through the collection of all Fix Released linux bugs and
    # determine which release a bug was fixed in
    while end <= total_fix_released:
        offset = 0 # offset of tasks into each collection slice
        try:
            tasks = total_tasks[start:end]
            for task in tasks:
                offset = offset + 1
                (date_fixed, err_count) = get_date_fixed(task, err_count)
                # determine which release the bug was fixed
                position = bisect.bisect(release_dates, date_fixed)
                if position >= len(release_dates):
                    continue
                total_fixed[position].append(str(task.bug.id))

            (start, end) = check_end(start, end, STEP, total_fix_released)
            if start == end:
                break
        except HTTPError, e:
            print e.content
            # reset start to (start + offset) to continue processing slice
            if offset != STEP:
                start = start + offset
                continue
            # slice had been fully processed, check if we've reached the
            # end of the entire collection, if not grab next slice
            else:
                (start, end) = check_end(start, end, STEP, total_fix_released)
                if start == end:
                    break

    create_file("fixed-linux-total.data", total_fixed)

    # reset start and end
    (start, end) = init_step(janitor_fix_released, STEP)
    err_count = 0
    janitor_kernel_fixed = 0

    # Cycle through the collection of Fix Released linux bugs which
    # the LP Janitor has specifically posted a comment.  Determine
    # if the Janitor Fix Released the bug and if so which release it belongs to
    while end <= janitor_fix_released:
        try:
            tasks = janitor_tasks[start:end]
            offset = 0
            for task in tasks:
                offset = offset + 1
                bug = task.bug
                # suffers from bug 279561 so catch the Exception and continue on
                try:
                   msgs = bug.messages_collection
                except HTTPError, e:
#                    print e.content
                    print "Error, suffer from bug 279561"
                    err_count = err_count + 1
                    continue
                (date_fixed, err_count) = get_date_fixed(task, err_count)
                # total number of comments in a bug
                remaining = msgs.total_size
                janitor_kernel_fixed = parse_msgs (msgs, remaining, bug, janitor, date_fixed, janitor_kernel_fixed)

            (start, end) = check_end(start, end, STEP, janitor_fix_released)
            if start == end:
                print "!!!!!!!  FINISHED janitor fixed %d !!!!!!!!!!" % (janitor_kernel_fixed)
                break
        except HTTPError, e:
            err_count = err_count + 1
            print "Error, reset slice"
#           print e.content
            # reset start to (start + offset) to continue processing slice
            if offset != STEP:
                start = start + offset
                continue
            # slice had been fully processed, check if we've reached the
            # end of the entire collection, if not grab next slice
            else:
                (start, end) = check_end(start, end, STEP, janitor_fix_released)
                if start == end:
                    break

    create_file("fixed-linux-known-patches.data", kernelteam_fixed)

main()
